//=======================================================
//
//  M8 USB with IR capture, 12MHz crystal
//
//  using obdev USB firmware, using portD, bit2 & bit7 
//
//
//=======================================================

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

/* ----- Hardware Config, remember to change the following definition in usbdrv.h 
         if different pins are used, please note additional requirement on D+ Pin ---

// #define USB_CFG_IOPORTNAME      D
    This is the port where the USB bus is connected. When you configure it to
    "B", the registers PORTB, PINB and DDRB will be used.

   #define USB_CFG_DMINUS_BIT      7
    This is the bit number in USB_CFG_IOPORT where the USB D- line is connected.
    This may be any bit in the port.

   #define USB_CFG_DPLUS_BIT       2
    This is the bit number in USB_CFG_IOPORT where the USB D+ line is connected.
    This may be any bit in the port. Please note that D+ MUST also be connected
    to interrupt pin INT0! (or simply use the INT0 pin for D+ instead)
*/


#include "usbdrv.h"

#ifndef uint
#define uint            unsigned int
#endif

#define getstata        1
#define getcount        2
#define getirdata       3
#define next_run        4

#define IR_ReadPort	    PORTB
#define IR_PortDDR      DDRB


//==== GLOBAL VARIABLES ========================== 

uint  irdata[244];
uchar irdataindex;
uchar readindex;
uchar busy;
uchar start;

//==== FUNCTION PROTOCAL =========================
uchar usbFunctionSetup(uchar data[8]);
uchar usbFunctionRead(uchar *data, uchar len);
void timer0_init(void);
void timer1_init(void);

//================================================
uchar usbFunctionSetup(uchar data[8])
{
    uchar len = 0;

    switch (data[1])
    {
    case getstata:
        usbMsgPtr = &busy;
        len       = 1;
        break;

    case getcount:
        usbMsgPtr = &irdataindex;
        len       = 1;
        break;

    case getirdata:
        readindex = 0;
        len       = 0xff;
        break;

    case next_run:
        irdataindex = 0;
        busy        = 0;
        start       = 1;
        break;

    default:
        break;
    }
    return(len);
}

//=================================================
uchar usbFunctionRead(uchar *data, uchar len)
{
    uchar i;

    for (i = 0; i<len/2; i++) //len need to be even
    {
        data[2 * i]     = (uchar)(irdata[readindex] >> 8);
        data[2 * i + 1] = (uchar)(irdata[readindex]);
        readindex++;
    }
    return(len);
}

//==================================================
//
void timer0_init(void)
{
    TCCR0 = 0x00;                    //stop counter
    TCNT0 = 0x50;                    //set 15ms
    TIMSK = TIMSK & (~(1 << TOIE0)); //no interrupt
    TIFR  = TIFR | (1 << TOV0);      //clear int flag
    TCCR0 = 0x05;                    //start timer with 1024 prescaler
}

//==================================================
//
void timer1_init(void)
{
    TCCR1B = 0x00;                                      //stop timer
    TIMSK  = TIMSK & (~((1 << TOIE1) | (1 << TICIE1))); //Timer1 input capture INT disable
    TIFR   = TIFR | (1 << TOV1);                        //clear T1 overflow flag
    TIFR   = TIFR | (1 << ICF1);                        //clear input capture flag
    TCNT1H = 0x00;
    TCNT1L = 0x00;                                      //starts from 0, 5.33uS per count
    TCCR1A = 0x00;
    TCCR1B = 0x83;                                      //start timer with 64 prescaler, noise surpress mode
}

//==================================================
int main(void)
{
    uchar        i, j;
    unsigned int itemp;

    IR_ReadPort = 0xff; /* activate all pull-ups on IR Read port */
 
    IR_PortDDR = 0;    /* all pins input */
 
    // USBIN & USBDDR defined in usbdrv.h

    USBOUT  = (uchar)~(( 1<<USB_CFG_DMINUS_BIT )|( 1<<USB_CFG_DPLUS_BIT ));  /* activate pull-ups except on USB lines */
    USBDDR  = ( 1<<USB_CFG_DMINUS_BIT )|( 1<<USB_CFG_DPLUS_BIT );     /* 1000 0100 bin: all pins input except USB (-> USB reset) */
 
    j     = 0;
    while (--j)
    {
        /* USB Reset by device only required on Watchdog Reset */
        i = 0;
        while (--i)
        {
            ;
        }           /* delay >10ms for USB reset */
    }
 
    USBDDR = 0;    /* all as inputs: remove USB reset condition */

    readindex   = 0;
    irdataindex = 0;
      
    DDRC = 0x3; 
    busy        = 0;
    start       = 1;

    PORTC = 0x02;

    TIFR        = TIFR | (1 << TOV0); //clear interrupt flag
    TIFR        = TIFR | (1 << TOV1); //clear T1 overflow flag
    TIFR        = TIFR | (1 << ICF1); //clear input capture flag

    usbInit();
    sei();

    for (;;)
    {
        /* main event loop */
        usbPoll();

        ////////////// Place user codes between these boundary ////////////////////////////////////////

        if ((busy == 0) && (start == 1)) //start once IR data capture
        {
            busy  = 1;                   //PC check this variable to see if capture ends
            start = 0;                   //PC must read IR data before next capture can start
            PORTC = 0x02;
  
            timer1_init();
        }
        if (TIFR & (1 << ICF1))   //with IR, save counts and initialise timer to 0, change capture edge
        {
            if (irdataindex == 0) //first capture, T1 starts from 0
            {
                TCNT1H    = 0x00;
                TCNT1L    = 0x00; //clear to start from 0
                irdata[0] = 0;
                irdataindex++;

                TIFR = TIFR | (1 << ICF1); //clear capture INT flag

                timer0_init();
            }
            else               //IR not first edge
            {
                itemp = ICR1L; //save IR timing
                itemp = itemp | (ICR1H << 8);
                irdata[irdataindex] = itemp;
                irdataindex++;

                TIFR = TIFR | (1 << ICF1); //clear input capture INT flag

                timer0_init();

                if (irdataindex > 244)     // index over-ranged 
                {
                  TCCR0  = 0x00;               //stop timer0
                  TIFR   = TIFR | (1 << TOV0); //clear interrupt flag
                  TCCR1B = 0x00;               //stop timer
                  busy   = 0;
                  PORTC = 0x1;
				} 
            }
      
	        TCCR1B ^= (1<<ICES1);             // toggle capture edge 
	     	  
	    }
        if (TIFR & (1 << TOV0))          //If T0>15ms, then IR signal ended
        {
            TCCR0  = 0x00;               //stop timer0
            TIFR   = TIFR | (1 << TOV0); //clear interrupt flag
            TCCR1B = 0x00;               //stop timer
            busy   = 0;
            
            PORTC = 0x1;
        }

  //////////////// End of User code  //////////////////////////////////////////////////////////////

    } // End of infinite FOR loop
    
    return(0);
}
